@@ -7,8 +7,6 @@
 
 #ifdef TSRM_WIN32
 # include "tsrm_config.w32.h"
-#elif defined(NETWARE)
-# include "tsrm_config.nw.h"
 #else
 # include <tsrm_config.h>
 # include <sys/param.h>
@@ -25,7 +23,9 @@
 #pragma alloca
 #  else
 #   ifndef alloca /* predefined by HP cc +Olibcalls */
+#    ifndef NETWARE
 char *alloca ();
+#    endif
 #   endif
 #  endif
 # endif
@@ -42,8 +42,6 @@ char *alloca ();
 #ifndef MAXPATHLEN
 # ifdef PATH_MAX
 #  define MAXPATHLEN PATH_MAX
-# elif defined(MAX_PATH)
-#  define MAXPATHLEN MAX_PATH
 # else
 #  define MAXPATHLEN 256
 # endif
