@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: posix.c,v 1.60 2004/04/18 21:49:10 iliaa Exp $ */
+/* $Id: posix.c,v 1.60.2.2 2005/01/28 01:39:35 tony2001 Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -131,7 +131,7 @@ function_entry posix_functions[] = {
 static PHP_MINFO_FUNCTION(posix)
 {
 	php_info_print_table_start();
-	php_info_print_table_row(2, "Revision", "$Revision: 1.60 $");
+	php_info_print_table_row(2, "Revision", "$Revision: 1.60.2.2 $");
 	php_info_print_table_end();
 }
 /* }}} */
@@ -180,7 +180,7 @@ ZEND_GET_MODULE(posix)
 
 #define PHP_POSIX_SINGLE_ARG_FUNC(func_name)	\
 	long val;	\
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &val) == FAILURE) return;	\
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &val) == FAILURE) RETURN_FALSE;	\
 	if (func_name(val) < 0) {	\
 		POSIX_G(last_error) = errno;	\
 		RETURN_FALSE;	\
@@ -194,8 +194,9 @@ PHP_FUNCTION(posix_kill)
 {
 	long pid, sig;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ll", &pid, &sig) == FAILURE)
-		return;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ll", &pid, &sig) == FAILURE) {
+		RETURN_FALSE;
+	}
   
 	if (kill(pid, sig) < 0) {
 		POSIX_G(last_error) = errno;
@@ -354,8 +355,9 @@ PHP_FUNCTION(posix_setpgid)
 {
 	long pid, pgid;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ll", &pid, &pgid) == FAILURE)
-		return;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ll", &pid, &pgid) == FAILURE) {
+		RETURN_FALSE;
+	}
 	
 	if (setpgid(pid, pgid) < 0) {
 		POSIX_G(last_error) = errno;
@@ -371,7 +373,16 @@ PHP_FUNCTION(posix_setpgid)
 #ifdef HAVE_GETPGID
 PHP_FUNCTION(posix_getpgid)
 {
-	PHP_POSIX_SINGLE_ARG_FUNC(getpgid);
+	long val;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &val) == FAILURE) {
+		RETURN_FALSE;
+	}
+	
+	if ((val = getpgid(val)) < 0) {
+		POSIX_G(last_error) = errno;
+		RETURN_FALSE;
+	}
+	RETURN_LONG(val);
 }
 #endif
 /* }}} */
@@ -381,7 +392,16 @@ PHP_FUNCTION(posix_getpgid)
 #ifdef HAVE_GETSID
 PHP_FUNCTION(posix_getsid)
 {
-	PHP_POSIX_SINGLE_ARG_FUNC(getsid);
+	long val;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &val) == FAILURE) {
+		RETURN_FALSE;
+	}
+	
+	if ((val = getsid(val)) < 0) {
+		POSIX_G(last_error) = errno;
+		RETURN_FALSE;
+	}
+	RETURN_LONG(val);
 }
 #endif
 /* }}} */
@@ -495,8 +515,9 @@ PHP_FUNCTION(posix_ttyname)
 	char *p;
 	int fd;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z", &z_fd) == FAILURE)
-		return;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z", &z_fd) == FAILURE) {
+		RETURN_FALSE;
+	}
 
 	switch (Z_TYPE_P(z_fd)) {
 		case IS_RESOURCE:
@@ -525,8 +546,9 @@ PHP_FUNCTION(posix_isatty)
 	zval *z_fd;
 	int fd;
 	
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z", &z_fd) == FAILURE)
-		return;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z", &z_fd) == FAILURE) {
+		RETURN_FALSE;
+	}
 
 	switch (Z_TYPE_P(z_fd)) {
 		case IS_RESOURCE:
@@ -591,8 +613,9 @@ PHP_FUNCTION(posix_mkfifo)
 	long mode;
 	int     result;
 	
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sl", &path, &path_len, &mode) == FAILURE)
-		return;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sl", &path, &path_len, &mode) == FAILURE) {
+		RETURN_FALSE;
+	}
 
 	if (PG(safe_mode) && (!php_checkuid(path, NULL, CHECKUID_ALLOW_ONLY_DIR))) {
 		RETURN_FALSE;
@@ -656,8 +679,9 @@ PHP_FUNCTION(posix_getgrnam)
 	struct group *g;
 	int name_len;
 	
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &name, &name_len) == FAILURE)
-		return;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &name, &name_len) == FAILURE) {
+		RETURN_FALSE;
+	}
 
 	if (NULL == (g = getgrnam(name))) {
 		POSIX_G(last_error) = errno;
@@ -680,8 +704,9 @@ PHP_FUNCTION(posix_getgrgid)
 	long gid;
 	struct group *g;
 	
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &gid) == FAILURE)
-		return;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &gid) == FAILURE) {
+		RETURN_FALSE;
+	}
 
 	if (NULL == (g = getgrgid(gid))) {
 		POSIX_G(last_error) = errno;
@@ -721,9 +746,9 @@ PHP_FUNCTION(posix_getpwnam)
 	char *name;
 	int name_len;
 	
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &name, &name_len) == FAILURE)
-		return;
-
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &name, &name_len) == FAILURE) {
+		RETURN_FALSE;
+	}
 	if (NULL == (pw = getpwnam(name))) {
 		POSIX_G(last_error) = errno;
 		RETURN_FALSE;
@@ -746,8 +771,9 @@ PHP_FUNCTION(posix_getpwuid)
 	long uid;
 	struct passwd *pw;
 	
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &uid) == FAILURE)
-		return;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &uid) == FAILURE) {
+		RETURN_FALSE;
+	}
 
 	if (NULL == (pw = getpwuid(uid))) {
 		POSIX_G(last_error) = errno;
@@ -895,8 +921,9 @@ PHP_FUNCTION(posix_strerror)
 {
 	long error;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &error) == FAILURE)
-		return;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &error) == FAILURE) {
+		RETURN_FALSE;
+	}
 
 	RETURN_STRING(strerror(error), 1);
 }
