@@ -20,7 +20,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: snmp.c,v 1.92.2.1 2004/12/07 00:33:58 iliaa Exp $ */
+/* $Id: snmp.c,v 1.92.2.5 2005/03/17 19:47:48 harrie Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -133,6 +133,12 @@ function_entry snmp_functions[] = {
 #endif
 	PHP_FE(snmpset, NULL)
 
+	PHP_FE(snmp2_get, NULL)
+	PHP_FE(snmp2_getnext, NULL)
+	PHP_FE(snmp2_walk, NULL)
+	PHP_FE(snmp2_real_walk, NULL)
+	PHP_FE(snmp2_set, NULL)
+
 	PHP_FE(snmp3_get, NULL)
 	PHP_FE(snmp3_getnext, NULL)
 	PHP_FE(snmp3_walk, NULL)
@@ -153,7 +159,7 @@ zend_module_entry snmp_module_entry = {
 	"snmp",
 	snmp_functions,
 	PHP_MINIT(snmp),
-	NULL,
+	PHP_MSHUTDOWN(snmp),
 	NULL,
 	NULL,
 	PHP_MINFO(snmp),
@@ -205,6 +211,16 @@ PHP_MINIT_FUNCTION(snmp)
 }
 /* }}} */
 
+/* {{{ PHP_MSHUTDOWN_FUNCTION
+ */
+PHP_MSHUTDOWN_FUNCTION(snmp)
+{
+	snmp_shutdown("snmpapp");
+
+	return SUCCESS;
+}
+/* }}} */
+
 /* {{{ PHP_MINFO_FUNCTION
  */
 PHP_MINFO_FUNCTION(snmp)
@@ -314,7 +330,7 @@ static void php_snmp_getvalue(struct variable_list *vars, zval *snmpval TSRMLS_D
 
 /* {{{ php_snmp_internal
 *
-* Generic SNMP object fetcher (for both v3 and v1)
+* Generic SNMP object fetcher (for all SNMP versions)
 *
 * st=1   snmpget()  - query an agent and return a single value.
 * st=2   snmpget()  - query an agent and return the next single value.
@@ -514,7 +530,7 @@ retry:
 
 /* {{{ php_snmp
 *
-* Generic SNMPv1 handler
+* Generic community based SNMP handler for version 1 and 2.
 * This function makes use of the internal SNMP object fetcher.
 * The object fetcher is shared with SNMPv3.
 *
@@ -528,7 +544,7 @@ retry:
 * st=11  snmpset() - query an agent and set a single value
 *
 */
-static void php_snmp(INTERNAL_FUNCTION_PARAMETERS, int st) 
+static void php_snmp(INTERNAL_FUNCTION_PARAMETERS, int st, int version) 
 {
 	zval **a1, **a2, **a3, **a4, **a5, **a6, **a7;
 	struct snmp_session session;
@@ -590,7 +606,7 @@ static void php_snmp(INTERNAL_FUNCTION_PARAMETERS, int st)
 
 	session.peername = hostname;
 	session.remote_port = remote_port;
-	session.version = SNMP_VERSION_1;
+	session.version = version;
 	/*
 	* FIXME: potential memory leak
 	* This is a workaround for an "artifact" (Mike Slifcak)
@@ -616,7 +632,7 @@ static void php_snmp(INTERNAL_FUNCTION_PARAMETERS, int st)
    Fetch a SNMP object */
 PHP_FUNCTION(snmpget)
 {
-	php_snmp(INTERNAL_FUNCTION_PARAM_PASSTHRU,1);
+	php_snmp(INTERNAL_FUNCTION_PARAM_PASSTHRU,1, SNMP_VERSION_1);
 }
 /* }}} */
 
@@ -624,7 +640,7 @@ PHP_FUNCTION(snmpget)
    Fetch a SNMP object */
 PHP_FUNCTION(snmpgetnext)
 {
-	php_snmp(INTERNAL_FUNCTION_PARAM_PASSTHRU,2);
+	php_snmp(INTERNAL_FUNCTION_PARAM_PASSTHRU,2, SNMP_VERSION_1);
 }
 /* }}} */
 
@@ -632,7 +648,7 @@ PHP_FUNCTION(snmpgetnext)
    Return all objects under the specified object id */
 PHP_FUNCTION(snmpwalk)
 {
-	php_snmp(INTERNAL_FUNCTION_PARAM_PASSTHRU,3);
+	php_snmp(INTERNAL_FUNCTION_PARAM_PASSTHRU,3, SNMP_VERSION_1);
 }
 /* }}} */
 
@@ -640,7 +656,7 @@ PHP_FUNCTION(snmpwalk)
    Return all objects including their respective object id withing the specified one */
 PHP_FUNCTION(snmprealwalk)
 {
-	php_snmp(INTERNAL_FUNCTION_PARAM_PASSTHRU,4);
+	php_snmp(INTERNAL_FUNCTION_PARAM_PASSTHRU,4, SNMP_VERSION_1);
 }
 /* }}} */
 
@@ -718,7 +734,7 @@ PHP_FUNCTION(snmp_set_oid_numeric_print)
    Set the value of a SNMP object */
 PHP_FUNCTION(snmpset)
 {
-	php_snmp(INTERNAL_FUNCTION_PARAM_PASSTHRU,11);
+	php_snmp(INTERNAL_FUNCTION_PARAM_PASSTHRU,11, SNMP_VERSION_1);
 }
 /* }}} */
 
@@ -800,6 +816,9 @@ static int netsnmp_session_set_sec_protocol(struct snmp_session *s, char *prot T
 * symbol on purpose, as it's defined to be the same as the former.
 */
 			|| !strcasecmp(prot, "AES")) {
+			s->securityPrivProto = usmAES128PrivProtocol;
+			s->securityPrivProtoLen = OIDSIZE(usmAES128PrivProtocol);
+			return (0);
 #else			
 		) {
 			s->securityPrivProto = usmAES128PrivProtocol;
@@ -834,7 +853,7 @@ static int netsnmp_session_gen_auth_key(struct snmp_session *s, char *pass TSRML
 		s->securityAuthKeyLen = USM_AUTH_KU_LEN;
 		if (s->securityAuthProto == NULL) {
 			/* get .conf set default */
-			oid *def = get_default_authtype(&(s->securityAuthProtoLen));
+			const oid *def = get_default_authtype(&(s->securityAuthProtoLen));
 			s->securityAuthProto = snmp_duplicate_objid(def, s->securityAuthProtoLen);
 		}
 		if (s->securityAuthProto == NULL) {
@@ -863,7 +882,7 @@ static int netsnmp_session_gen_sec_key(struct snmp_session *s, u_char *pass TSRM
 		s->securityPrivKeyLen = USM_PRIV_KU_LEN;
 		if (s->securityPrivProto == NULL) {
 			/* get .conf set default */
-			oid *def = get_default_privtype(&(s->securityPrivProtoLen));
+			const oid *def = get_default_privtype(&(s->securityPrivProtoLen));
 			s->securityPrivProto = snmp_duplicate_objid(def, s->securityPrivProtoLen);
 		}
 		if (s->securityPrivProto == NULL) {
@@ -884,6 +903,45 @@ static int netsnmp_session_gen_sec_key(struct snmp_session *s, u_char *pass TSRM
 }
 /* }}} */
 
+/* {{{ proto string snmp2_get(string host, string community, string object_id [, int timeout [, int retries]]) 
+   Fetch a SNMP object */
+PHP_FUNCTION(snmp2_get)
+{
+	php_snmp(INTERNAL_FUNCTION_PARAM_PASSTHRU,1, SNMP_VERSION_2c);
+}
+/* }}} */
+
+/* {{{ proto string snmp2_getnext(string host, string community, string object_id [, int timeout [, int retries]]) 
+   Fetch a SNMP object */
+PHP_FUNCTION(snmp2_getnext)
+{
+	php_snmp(INTERNAL_FUNCTION_PARAM_PASSTHRU,2, SNMP_VERSION_2c);
+}
+/* }}} */
+
+/* {{{ proto array snmp2_walk(string host, string community, string object_id [, int timeout [, int retries]]) 
+   Return all objects under the specified object id */
+PHP_FUNCTION(snmp2_walk)
+{
+	php_snmp(INTERNAL_FUNCTION_PARAM_PASSTHRU,3, SNMP_VERSION_2c);
+}
+/* }}} */
+
+/* {{{ proto array snmp2_real_walk(string host, string community, string object_id [, int timeout [, int retries]])
+   Return all objects including their respective object id withing the specified one */
+PHP_FUNCTION(snmp2_real_walk)
+{
+	php_snmp(INTERNAL_FUNCTION_PARAM_PASSTHRU,4, SNMP_VERSION_2c);
+}
+/* }}} */
+
+/* {{{ proto int snmp2_set(string host, string community, string object_id, string type, mixed value [, int timeout [, int retries]]) 
+   Set the value of a SNMP object */
+PHP_FUNCTION(snmp2_set)
+{
+	php_snmp(INTERNAL_FUNCTION_PARAM_PASSTHRU,11, SNMP_VERSION_2c);
+}
+/* }}} */
 
 /* {{{ proto void php_snmpv3(INTERNAL_FUNCTION_PARAMETERS, int st)
 *
