@@ -17,10 +17,11 @@
   |          Dmitry Stogov <dmitry@zend.com>                             |
   +----------------------------------------------------------------------+
 */
-/* $Id: php_http.c,v 1.55.2.5 2004/12/01 18:22:24 dmitry Exp $ */
+/* $Id: php_http.c,v 1.55.2.14 2005/03/23 08:08:34 dmitry Exp $ */
 
 #include "php_soap.h"
 #include "ext/standard/base64.h"
+#include "ext/standard/md5.h"
 
 static char *get_http_header_value(char *headers, char *type);
 static int get_http_body(php_stream *socketd, int close, char *headers,  char **response, int *out_size TSRMLS_DC);
@@ -81,14 +82,17 @@ static void proxy_authentication(zval* this_ptr, smart_str* soap_headers TSRMLS_
 static php_stream* http_connect(zval* this_ptr, php_url *phpurl, int use_ssl, int *use_proxy TSRMLS_DC)
 {
 	php_stream *stream;
-	zval **proxy_host, **proxy_port;
+	zval **proxy_host, **proxy_port, **tmp;
 	char *host;
 #ifdef ZEND_ENGINE_2
+	php_stream_context *context = NULL;
 	char *name;
 	long namelen;
 #endif
 	int port;
 	int old_error_reporting;
+	struct timeval tv;
+	struct timeval *timeout = NULL;
 
 	if (zend_hash_find(Z_OBJPROP_P(this_ptr), "_proxy_host", sizeof("_proxy_host"), (void **) &proxy_host) == SUCCESS &&
 	    Z_TYPE_PP(proxy_host) == IS_STRING &&
@@ -101,21 +105,41 @@ static php_stream* http_connect(zval* this_ptr, php_url *phpurl, int use_ssl, in
 		host = phpurl->host;
 		port = phpurl->port;
 	}
+	if (zend_hash_find(Z_OBJPROP_P(this_ptr), "_connection_timeout", sizeof("_connection_timeout"), (void **) &tmp) == SUCCESS &&
+	    Z_TYPE_PP(tmp) == IS_LONG && Z_LVAL_PP(tmp) > 0) {
+	  tv.tv_sec = Z_LVAL_PP(tmp);
+	  tv.tv_usec = 0;
+		timeout = &tv;
+	}
 
 	old_error_reporting = EG(error_reporting);
 	EG(error_reporting) &= ~(E_WARNING|E_NOTICE|E_USER_WARNING|E_USER_NOTICE);
 
 #ifdef ZEND_ENGINE_2
 	namelen = spprintf(&name, 0, "%s://%s:%d", (use_ssl && !*use_proxy)? "ssl" : "tcp", host, port);
+	if (use_ssl) {
+		zval **tmp;
+
+		if (zend_hash_find(Z_OBJPROP_P(this_ptr), "_local_cert", sizeof("_local_cert"), (void **) &tmp) == SUCCESS &&
+		    Z_TYPE_PP(tmp) == IS_STRING) {
+			context = php_stream_context_alloc();
+			php_stream_context_set_option(context, "ssl", "local_cert", *tmp);
+			if (zend_hash_find(Z_OBJPROP_P(this_ptr), "_passphrase", sizeof("_passphrase"), (void **) &tmp) == SUCCESS &&
+			    Z_TYPE_PP(tmp) == IS_STRING) {
+				php_stream_context_set_option(context, "ssl", "passphrase", *tmp);
+			}
+		}
+	}
 	stream = php_stream_xport_create(name, namelen,
 		ENFORCE_SAFE_MODE | REPORT_ERRORS,
 		STREAM_XPORT_CLIENT | STREAM_XPORT_CONNECT,
 		NULL /*persistent_id*/,
-		NULL /*timeout*/,
-		NULL, NULL, NULL);
+		timeout,
+		context,
+		NULL, NULL);
 	efree(name);
 #else
-	stream = php_stream_sock_open_host(host, port, SOCK_STREAM, NULL, NULL);
+	stream = php_stream_sock_open_host(host, port, SOCK_STREAM, timeout, NULL);
 #endif
 
 	/* SSL & proxy */
@@ -184,13 +208,13 @@ static int in_domain(const char *host, const char *domain)
   }
 }
 
-int make_http_soap_request(zval  *this_ptr, 
+int make_http_soap_request(zval  *this_ptr,
                            char  *buf,
                            int    buf_size,
-                           char  *location, 
-                           char  *soapaction, 
+                           char  *location,
+                           char  *soapaction,
                            int    soap_version,
-                           char **buffer, 
+                           char **buffer,
                            int   *buffer_len TSRMLS_DC)
 {
 	char *request;
@@ -271,7 +295,7 @@ int make_http_soap_request(zval  *this_ptr,
 		phpurl = php_url_parse(location);
 	}
 
-try_again:	
+try_again:
 	if (phpurl == NULL || phpurl->host == NULL) {
 	  if (phpurl != NULL) {php_url_free(phpurl);}
 		if (request != buf) {efree(request);}
@@ -412,23 +436,164 @@ try_again:
 		smart_str_append_const(&soap_headers, "\r\n");
 
 		/* HTTP Authentication */
-		if (zend_hash_find(Z_OBJPROP_P(this_ptr), "_login", sizeof("_login"), (void **)&login) == SUCCESS) {
-			char* buf;
-			int len;
+		if (zend_hash_find(Z_OBJPROP_P(this_ptr), "_login", sizeof("_login"), (void **)&login) == SUCCESS &&
+		    Z_TYPE_PP(login) == IS_STRING) {
+			zval **digest;
 
-			smart_str auth = {0};
-			smart_str_appendl(&auth, Z_STRVAL_PP(login), Z_STRLEN_PP(login));
-			smart_str_appendc(&auth, ':');
-			if (zend_hash_find(Z_OBJPROP_P(this_ptr), "_password", sizeof("_password"), (void **)&password) == SUCCESS) {
-				smart_str_appendl(&auth, Z_STRVAL_PP(password), Z_STRLEN_PP(password));
+			if (zend_hash_find(Z_OBJPROP_P(this_ptr), "_digest", sizeof("_digest"), (void **)&digest) == SUCCESS) {
+				if (Z_TYPE_PP(digest) == IS_ARRAY) {
+					char          HA1[33], HA2[33], response[33], cnonce[33], nc[9];
+					PHP_MD5_CTX   md5ctx;
+					unsigned char hash[16];
+
+					PHP_MD5Init(&md5ctx);
+					sprintf(cnonce, "%d", rand());
+					PHP_MD5Update(&md5ctx, cnonce, strlen(cnonce));
+					PHP_MD5Final(hash, &md5ctx);
+					make_digest(cnonce, hash);
+
+					if (zend_hash_find(Z_ARRVAL_PP(digest), "nc", sizeof("nc"), (void **)&tmp) == SUCCESS &&
+					    Z_TYPE_PP(tmp) == IS_LONG) {
+						Z_LVAL_PP(tmp)++;
+						sprintf(nc, "%08ld", Z_LVAL_PP(tmp));
+					} else {
+						add_assoc_long(*digest, "nc", 1);
+						strcpy(nc, "00000001");
+					}
+
+					PHP_MD5Init(&md5ctx);
+					PHP_MD5Update(&md5ctx, Z_STRVAL_PP(login), Z_STRLEN_PP(login));
+					PHP_MD5Update(&md5ctx, ":", 1);
+					if (zend_hash_find(Z_ARRVAL_PP(digest), "realm", sizeof("realm"), (void **)&tmp) == SUCCESS &&
+					    Z_TYPE_PP(tmp) == IS_STRING) {
+						PHP_MD5Update(&md5ctx, Z_STRVAL_PP(tmp), Z_STRLEN_PP(tmp));
+					}
+					PHP_MD5Update(&md5ctx, ":", 1);
+					if (zend_hash_find(Z_OBJPROP_P(this_ptr), "_password", sizeof("_password"), (void **)&password) == SUCCESS &&
+					    Z_TYPE_PP(password) == IS_STRING) {
+						PHP_MD5Update(&md5ctx, Z_STRVAL_PP(password), Z_STRLEN_PP(password));
+					}
+					PHP_MD5Final(hash, &md5ctx);
+					make_digest(HA1, hash);
+					if (zend_hash_find(Z_ARRVAL_PP(digest), "algorithm", sizeof("algorithm"), (void **)&tmp) == SUCCESS &&
+					    Z_TYPE_PP(tmp) == IS_STRING &&
+					    Z_STRLEN_PP(tmp) == sizeof("md5-sess")-1 &&
+					    stricmp(Z_STRVAL_PP(tmp), "md5-sess") == 0) {
+						PHP_MD5Init(&md5ctx);
+						PHP_MD5Update(&md5ctx, HA1, 32);
+						PHP_MD5Update(&md5ctx, ":", 1);
+						if (zend_hash_find(Z_ARRVAL_PP(digest), "nonce", sizeof("nonce"), (void **)&tmp) == SUCCESS &&
+						    Z_TYPE_PP(tmp) == IS_STRING) {
+							PHP_MD5Update(&md5ctx, Z_STRVAL_PP(tmp), Z_STRLEN_PP(tmp));
+						}
+						PHP_MD5Update(&md5ctx, ":", 1);
+						PHP_MD5Update(&md5ctx, cnonce, 8);
+						PHP_MD5Final(hash, &md5ctx);
+						make_digest(HA1, hash);
+					}
+
+					PHP_MD5Init(&md5ctx);
+					PHP_MD5Update(&md5ctx, "POST:", sizeof("POST:")-1);
+					if (phpurl->path) {
+						PHP_MD5Update(&md5ctx, phpurl->path, strlen(phpurl->path));
+					}
+					if (phpurl->query) {
+						PHP_MD5Update(&md5ctx, "?", 1);
+						PHP_MD5Update(&md5ctx, phpurl->query, strlen(phpurl->query));
+					}
+
+					/* TODO: Support for qop="auth-int" */
+/*
+					if (zend_hash_find(Z_ARRVAL_PP(digest), "qop", sizeof("qop"), (void **)&tmp) == SUCCESS &&
+					    Z_TYPE_PP(tmp) == IS_STRING &&
+					    Z_STRLEN_PP(tmp) == sizeof("auth-int")-1 &&
+					    stricmp(Z_STRVAL_PP(tmp), "auth-int") == 0) {
+						PHP_MD5Update(&md5ctx, ":", 1);
+						PHP_MD5Update(&md5ctx, HEntity, HASHHEXLEN);
+					}
+*/
+					PHP_MD5Final(hash, &md5ctx);
+					make_digest(HA2, hash);
+
+					PHP_MD5Init(&md5ctx);
+					PHP_MD5Update(&md5ctx, HA1, 32);
+					PHP_MD5Update(&md5ctx, ":", 1);
+					if (zend_hash_find(Z_ARRVAL_PP(digest), "nonce", sizeof("nonce"), (void **)&tmp) == SUCCESS &&
+					    Z_TYPE_PP(tmp) == IS_STRING) {
+						PHP_MD5Update(&md5ctx, Z_STRVAL_PP(tmp), Z_STRLEN_PP(tmp));
+					}
+					PHP_MD5Update(&md5ctx, ":", 1);
+					if (zend_hash_find(Z_ARRVAL_PP(digest), "qop", sizeof("qop"), (void **)&tmp) == SUCCESS &&
+					    Z_TYPE_PP(tmp) == IS_STRING) {
+						PHP_MD5Update(&md5ctx, nc, 8);
+						PHP_MD5Update(&md5ctx, ":", 1);
+						PHP_MD5Update(&md5ctx, cnonce, 8);
+						PHP_MD5Update(&md5ctx, ":", 1);
+						/* TODO: Support for qop="auth-int" */
+						PHP_MD5Update(&md5ctx, "auth", sizeof("auth")-1);
+						PHP_MD5Update(&md5ctx, ":", 1);
+					}
+					PHP_MD5Update(&md5ctx, HA2, 32);
+					PHP_MD5Final(hash, &md5ctx);
+					make_digest(response, hash);
+	
+					smart_str_append_const(&soap_headers, "Authorization: Digest username=\"");
+					smart_str_appendl(&soap_headers, Z_STRVAL_PP(login), Z_STRLEN_PP(login));
+					if (zend_hash_find(Z_ARRVAL_PP(digest), "realm", sizeof("realm"), (void **)&tmp) == SUCCESS &&
+					    Z_TYPE_PP(tmp) == IS_STRING) {
+						smart_str_append_const(&soap_headers, "\", realm=\"");
+						smart_str_appendl(&soap_headers, Z_STRVAL_PP(tmp), Z_STRLEN_PP(tmp));
+					}
+					if (zend_hash_find(Z_ARRVAL_PP(digest), "nonce", sizeof("nonce"), (void **)&tmp) == SUCCESS &&
+					    Z_TYPE_PP(tmp) == IS_STRING) {
+						smart_str_append_const(&soap_headers, "\", nonce=\"");
+						smart_str_appendl(&soap_headers, Z_STRVAL_PP(tmp), Z_STRLEN_PP(tmp));
+					}
+					smart_str_append_const(&soap_headers, "\", uri=\"");
+					if (phpurl->path) {
+						smart_str_appends(&soap_headers, phpurl->path);
+					}
+					if (phpurl->query) {
+						smart_str_appendc(&soap_headers, '?');
+						smart_str_appends(&soap_headers, phpurl->query);
+					}
+					if (zend_hash_find(Z_ARRVAL_PP(digest), "qop", sizeof("qop"), (void **)&tmp) == SUCCESS &&
+					    Z_TYPE_PP(tmp) == IS_STRING) {
+					/* TODO: Support for qop="auth-int" */
+						smart_str_append_const(&soap_headers, "\", qop=\"auth");
+						smart_str_append_const(&soap_headers, "\", nc=\"");
+						smart_str_appendl(&soap_headers, nc, 8);
+						smart_str_append_const(&soap_headers, "\", cnonce=\"");
+						smart_str_appendl(&soap_headers, cnonce, 8);
+					}
+					smart_str_append_const(&soap_headers, "\", response=\"");
+					smart_str_appendl(&soap_headers, response, 32);
+					if (zend_hash_find(Z_ARRVAL_PP(digest), "opaque", sizeof("opaque"), (void **)&tmp) == SUCCESS &&
+					    Z_TYPE_PP(tmp) == IS_STRING) {
+						smart_str_append_const(&soap_headers, "\", opaque=\"");
+						smart_str_appendl(&soap_headers, Z_STRVAL_PP(tmp), Z_STRLEN_PP(tmp));
+					}
+					smart_str_append_const(&soap_headers, "\"\r\n");
+				}
+			} else {
+				char* buf;
+				int len;
+
+				smart_str auth = {0};
+				smart_str_appendl(&auth, Z_STRVAL_PP(login), Z_STRLEN_PP(login));
+				smart_str_appendc(&auth, ':');
+				if (zend_hash_find(Z_OBJPROP_P(this_ptr), "_password", sizeof("_password"), (void **)&password) == SUCCESS &&
+				    Z_TYPE_PP(password) == IS_STRING) {
+					smart_str_appendl(&auth, Z_STRVAL_PP(password), Z_STRLEN_PP(password));
+				}
+				smart_str_0(&auth);
+				buf = php_base64_encode(auth.c, auth.len, &len);
+				smart_str_append_const(&soap_headers, "Authorization: Basic ");
+				smart_str_appendl(&soap_headers, buf, len);
+				smart_str_append_const(&soap_headers, "\r\n");
+				efree(buf);
+				smart_str_free(&auth);
 			}
-			smart_str_0(&auth);
-			buf = php_base64_encode(auth.c, auth.len, &len);
-			smart_str_append_const(&soap_headers, "Authorization: Basic ");
-			smart_str_appendl(&soap_headers, buf, len);
-			smart_str_append_const(&soap_headers, "\r\n");
-			efree(buf);
-			smart_str_free(&auth);
 		}
 
 		/* Proxy HTTP Authentication */
@@ -456,10 +621,10 @@ try_again:
 						if (zend_hash_index_find(Z_ARRVAL_PP(data), 0, (void**)&value) == SUCCESS &&
 						    Z_TYPE_PP(value) == IS_STRING) {
 						  zval **tmp;
-						  if (zend_hash_index_find(Z_ARRVAL_PP(data), 1, (void**)&tmp) == SUCCESS &&
-						      strncmp(phpurl->path?phpurl->path:"/",Z_STRVAL_PP(tmp),Z_STRLEN_PP(tmp)) == 0 &&
-						      zend_hash_index_find(Z_ARRVAL_PP(data), 2, (void**)&tmp) == SUCCESS &&
-						      in_domain(phpurl->host,Z_STRVAL_PP(tmp)) &&
+						  if ((zend_hash_index_find(Z_ARRVAL_PP(data), 1, (void**)&tmp) == FAILURE ||
+						       strncmp(phpurl->path?phpurl->path:"/",Z_STRVAL_PP(tmp),Z_STRLEN_PP(tmp)) == 0) &&
+						      (zend_hash_index_find(Z_ARRVAL_PP(data), 2, (void**)&tmp) == FAILURE ||
+						       in_domain(phpurl->host,Z_STRVAL_PP(tmp))) &&
 						      (use_ssl || zend_hash_index_find(Z_ARRVAL_PP(data), 3, (void**)&tmp) == FAILURE)) {
 								smart_str_appendl(&soap_headers, key, strlen(key));
 								smart_str_appendc(&soap_headers, '=');
@@ -497,111 +662,46 @@ try_again:
 
 	}
 
-	if (!get_http_headers(stream, &http_headers, &http_header_size TSRMLS_CC)) {
-		if (request != buf) {efree(request);}
-		php_stream_close(stream);
-		zend_hash_del(Z_OBJPROP_P(this_ptr), "httpsocket", sizeof("httpsocket"));
-		zend_hash_del(Z_OBJPROP_P(this_ptr), "_use_proxy", sizeof("_use_proxy"));
-		add_soap_fault(this_ptr, "HTTP", "Error Fetching http headers", NULL, NULL TSRMLS_CC);
-		return FALSE;
-	}
-
-	if (zend_hash_find(Z_OBJPROP_P(this_ptr), "trace", sizeof("trace"), (void **) &trace) == SUCCESS &&
-	    Z_LVAL_PP(trace) > 0) {
-		add_property_stringl(this_ptr, "__last_response_headers", http_headers, http_header_size, 1);
-	}
-
-	/* Check to see what HTTP status was sent */
-	http_1_1 = 0;
-	http_status = 0;
-	http_version = get_http_header_value(http_headers,"HTTP/");
-	if (http_version) {
-		char *tmp;
-
-		if (strncmp(http_version,"1.1", 3)) {
-			http_1_1 = 1;
-		}
-
-		tmp = strstr(http_version," ");
-		if (tmp != NULL) {
-			tmp++;
-			http_status = atoi(tmp);
-		}
-		efree(http_version);
-
-		/* Process HTTP status codes */
-		if (http_status >= 200 && http_status < 300) {
-		} else if (http_status >= 300 && http_status < 400) {
-			char *loc;
-
-			if ((loc = get_http_header_value(http_headers,"Location: ")) != NULL) {
-			  php_url *new_url  = php_url_parse(loc);
-			  char *body;
-			  int body_size;
-
-				if (new_url != NULL) {
-					if (get_http_body(stream, !http_1_1, http_headers, &body, &body_size TSRMLS_CC)) {
-						efree(body);
-					} else {
-						php_stream_close(stream);
-						zend_hash_del(Z_OBJPROP_P(this_ptr), "httpsocket", sizeof("httpsocket"));
-						stream = NULL;
-					}
-					efree(http_headers);
-					efree(loc);
-					if (new_url->scheme == NULL && new_url->path != NULL) {						
-					  new_url->scheme = estrdup(phpurl->scheme);
-					  new_url->host = estrdup(phpurl->host);
-					  new_url->port = phpurl->port;
-						if (new_url->path && new_url->path[0] != '/') {
-						  char *t = phpurl->path?phpurl->path:"/";
-						  char *p = strrchr(t, '/');
-						  char *s = emalloc((p - t) + strlen(new_url->path) + 2);
-
-						  strncpy(s, t, (p - t) + 1);
-						  s[(p - t) + 1] = 0;
-						  strcat(s, new_url->path);
-						  efree(new_url->path);
-						  new_url->path = s;
-						} 
-					}
-					phpurl = new_url;
-					goto try_again;
-				}
-			}
-/*
-		} else if (http_status == 400) {
-			add_soap_fault(this_ptr, "HTTP", "Bad Request", NULL, NULL TSRMLS_CC);
-		} else if (http_status == 401) {
-			add_soap_fault(this_ptr, "HTTP", "Unauthorized Request", NULL, NULL TSRMLS_CC);
-		} else if (http_status == 405) {
-			add_soap_fault(this_ptr, "HTTP", "Method not allowed", NULL, NULL TSRMLS_CC);
-		} else if (http_status == 415) {
-			add_soap_fault(this_ptr, "HTTP", "Unsupported Media Type", NULL, NULL TSRMLS_CC);
-		} else if (http_status >= 400 && http_status < 500) {
-			add_soap_fault(this_ptr, "HTTP", "Client Error", NULL, NULL TSRMLS_CC);
-		} else if (http_status == 500) {
-			add_soap_fault(this_ptr, "HTTP", "Internal Server Error", NULL, NULL TSRMLS_CC);
-		} else if (http_status >= 500 && http_status < 600) {
-			add_soap_fault(this_ptr, "HTTP", "Server Error", NULL, NULL TSRMLS_CC);
-		} else {
-			add_soap_fault(this_ptr, "HTTP", "Unsupported HTTP status code", NULL, NULL TSRMLS_CC);
-*/
-		}
-
-		/* Try and get headers again */
-		if (http_status == 100) {
+	do {
+		if (!get_http_headers(stream, &http_headers, &http_header_size TSRMLS_CC)) {
 			efree(http_headers);
-			if (!get_http_headers(stream, &http_headers, &http_header_size TSRMLS_CC)) {
-				if (request != buf) {efree(request);}
-				php_stream_close(stream);
-				zend_hash_del(Z_OBJPROP_P(this_ptr), "httpsocket", sizeof("httpsocket"));
-				zend_hash_del(Z_OBJPROP_P(this_ptr), "_use_proxy", sizeof("_use_proxy"));
-				add_soap_fault(this_ptr, "HTTP", "Error Fetching http headers", NULL, NULL TSRMLS_CC);
-				return FALSE;
+			if (request != buf) {efree(request);}
+			php_stream_close(stream);
+			zend_hash_del(Z_OBJPROP_P(this_ptr), "httpsocket", sizeof("httpsocket"));
+			zend_hash_del(Z_OBJPROP_P(this_ptr), "_use_proxy", sizeof("_use_proxy"));
+			add_soap_fault(this_ptr, "HTTP", "Error Fetching http headers", NULL, NULL TSRMLS_CC);
+			return FALSE;
+		}
+
+		if (zend_hash_find(Z_OBJPROP_P(this_ptr), "trace", sizeof("trace"), (void **) &trace) == SUCCESS &&
+		    Z_LVAL_PP(trace) > 0) {
+			add_property_stringl(this_ptr, "__last_response_headers", http_headers, http_header_size, 1);
+		}
+
+		/* Check to see what HTTP status was sent */
+		http_1_1 = 0;
+		http_status = 0;
+		http_version = get_http_header_value(http_headers,"HTTP/");
+		if (http_version) {
+			char *tmp;
+
+			if (strncmp(http_version,"1.1", 3)) {
+				http_1_1 = 1;
+			}
+
+			tmp = strstr(http_version," ");
+			if (tmp != NULL) {
+				tmp++;
+				http_status = atoi(tmp);
+			}
+			efree(http_version);
+
+			/* Try and get headers again */
+			if (http_status == 100) {
+				efree(http_headers);
 			}
 		}
-	}
+	} while (http_status == 100);
 
 	/* Grab and send back every cookie */
 
@@ -728,6 +828,127 @@ try_again:
 		zend_hash_del(Z_OBJPROP_P(this_ptr), "_use_proxy", sizeof("_use_proxy"));
 	}
 
+	/* Process HTTP status codes */
+	if (http_status >= 300 && http_status < 400) {
+		char *loc;
+
+		if ((loc = get_http_header_value(http_headers,"Location: ")) != NULL) {
+			php_url *new_url  = php_url_parse(loc);
+			char *body;
+			int body_size;
+
+			if (new_url != NULL) {
+				if (get_http_body(stream, !http_1_1, http_headers, &body, &body_size TSRMLS_CC)) {
+					efree(body);
+				} else {
+					php_stream_close(stream);
+					zend_hash_del(Z_OBJPROP_P(this_ptr), "httpsocket", sizeof("httpsocket"));
+					stream = NULL;
+				}
+				efree(http_headers);
+				efree(http_body);
+				efree(loc);
+				if (new_url->scheme == NULL && new_url->path != NULL) {
+					new_url->scheme = estrdup(phpurl->scheme);
+					new_url->host = estrdup(phpurl->host);
+					new_url->port = phpurl->port;
+					if (new_url->path && new_url->path[0] != '/') {
+						char *t = phpurl->path?phpurl->path:"/";
+						char *p = strrchr(t, '/');
+						char *s = emalloc((p - t) + strlen(new_url->path) + 2);
+
+						strncpy(s, t, (p - t) + 1);
+						s[(p - t) + 1] = 0;
+						strcat(s, new_url->path);
+						efree(new_url->path);
+						new_url->path = s;
+					}
+				}
+				phpurl = new_url;
+
+				goto try_again;
+			}
+		}
+	} else if (http_status == 401) {
+		/* Digest authentication */
+		zval **digest, **login, **password;
+		char *auth = get_http_header_value(http_headers, "WWW-Authenticate: ");
+
+		if (auth &&
+				strstr(auth, "Digest") == auth &&
+		    (zend_hash_find(Z_OBJPROP_P(this_ptr), "_digest", sizeof("_digest"), (void **)&digest) == FAILURE ||
+		     Z_TYPE_PP(digest) != IS_ARRAY) &&
+		    zend_hash_find(Z_OBJPROP_P(this_ptr), "_login", sizeof("_login"), (void **)&login) == SUCCESS &&
+		    Z_TYPE_PP(login) == IS_STRING &&
+		    zend_hash_find(Z_OBJPROP_P(this_ptr), "_password", sizeof("_password"), (void **)&password) == SUCCESS &&
+		    Z_TYPE_PP(password) == IS_STRING) {
+			char *s;
+			zval *digest = NULL;
+
+			s = auth + sizeof("Digest")-1;
+			while (*s != '\0') {
+				char *name, *val;
+				while (*s == ' ') ++s;
+				name = s;
+				while (*s != '\0' && *s != '=') ++s;
+				if (*s == '=') {
+					*s = '\0';
+					++s;
+					if (*s == '"') {
+						++s;
+						val = s;
+						while (*s != '\0' && *s != '"') ++s;
+					} else {
+						val = s;
+						while (*s != '\0' && *s != ' ' && *s != ',') ++s;
+					}
+					if (*s != '\0') {
+						if (*s != ',') {
+							*s = '\0';
+							++s;
+							while (*s != '\0' && *s != ',') ++s;
+							if (*s != '\0') ++s;
+						} else {
+							*s = '\0';
+							++s;
+						}
+					}
+					if (digest == NULL) {
+						ALLOC_INIT_ZVAL(digest);
+						array_init(digest);
+					}
+					add_assoc_string(digest, name, val ,1);
+				}
+			}
+
+			if (digest != NULL) {
+				php_url *new_url  = emalloc(sizeof(php_url));
+
+#ifdef ZEND_ENGINE_2
+				digest->refcount--;
+#endif
+				add_property_zval_ex(this_ptr, "_digest", sizeof("_digest"), digest TSRMLS_CC);
+
+				*new_url = *phpurl;
+				if (phpurl->scheme) phpurl->scheme = estrdup(phpurl->scheme);
+				if (phpurl->user) phpurl->user = estrdup(phpurl->user);
+				if (phpurl->pass) phpurl->pass = estrdup(phpurl->pass);
+				if (phpurl->host) phpurl->host = estrdup(phpurl->host);
+				if (phpurl->path) phpurl->path = estrdup(phpurl->path);
+				if (phpurl->query) phpurl->query = estrdup(phpurl->query);
+				if (phpurl->fragment) phpurl->fragment = estrdup(phpurl->fragment);
+				phpurl = new_url;
+
+				efree(auth);
+				efree(http_headers);
+				efree(http_body);
+
+				goto try_again;
+			}
+		}
+		if (auth) efree(auth);
+	}
+
 	/* Check and see if the server even sent a xml document */
 	content_type = get_http_header_value(http_headers,"Content-Type: ");
 	if (content_type) {
@@ -804,6 +1025,46 @@ try_again:
 	}
 
 	efree(http_headers);
+
+	if (http_status >= 400) {
+		int error = 0;
+
+		if (*buffer_len == 0) {
+			error = 1;
+		} else if (*buffer_len > 0) {
+			char *s = *buffer;
+
+			while (*s != '\0' && *s < ' ') {
+			  s++;
+			}
+			if (strncmp(s, "<?xml", 5)) {
+			  error = 1;
+			}
+		}
+
+		if (error) {
+			efree(*buffer);
+			if (http_status == 400) {
+				add_soap_fault(this_ptr, "HTTP", "Bad Request", NULL, NULL TSRMLS_CC);
+			} else if (http_status == 401) {
+				add_soap_fault(this_ptr, "HTTP", "Unauthorized Request", NULL, NULL TSRMLS_CC);
+			} else if (http_status == 405) {
+				add_soap_fault(this_ptr, "HTTP", "Method not allowed", NULL, NULL TSRMLS_CC);
+			} else if (http_status == 415) {
+				add_soap_fault(this_ptr, "HTTP", "Unsupported Media Type", NULL, NULL TSRMLS_CC);
+			} else if (http_status >= 400 && http_status < 500) {
+				add_soap_fault(this_ptr, "HTTP", "Client Error", NULL, NULL TSRMLS_CC);
+			} else if (http_status == 500) {
+				add_soap_fault(this_ptr, "HTTP", "Internal Server Error", NULL, NULL TSRMLS_CC);
+			} else if (http_status >= 500 && http_status < 600) {
+				add_soap_fault(this_ptr, "HTTP", "Server Error", NULL, NULL TSRMLS_CC);
+			} else {
+				add_soap_fault(this_ptr, "HTTP", "Unsupported HTTP status code", NULL, NULL TSRMLS_CC);
+			}
+			return FALSE;
+		}
+	}
+
 	return TRUE;
 }
 
@@ -851,13 +1112,13 @@ static int get_http_body(php_stream *stream, int close, char *headers,  char **r
 	if (!close) {
 		header = get_http_header_value(headers, "Connection: ");
 		if (header) {
-			if(!strncmp(header, "close", sizeof("close")-1)) header_close = 1;
+			if(!strncasecmp(header, "close", sizeof("close")-1)) header_close = 1;
 			efree(header);
 		}
 	}
 	header = get_http_header_value(headers, "Transfer-Encoding: ");
 	if (header) {
-		if(!strncmp(header, "chunked", sizeof("chunked")-1)) header_chunked = 1;
+		if(!strncasecmp(header, "chunked", sizeof("chunked")-1)) header_chunked = 1;
 		efree(header);
 	}
 	header = get_http_header_value(headers, "Content-Length: ");
