@@ -1,13 +1,13 @@
 /* 
    +----------------------------------------------------------------------+
-   | PHP Version 4                                                        |
+   | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2006 The PHP Group                                |
+   | Copyright (c) 1997-2004 The PHP Group                                |
    +----------------------------------------------------------------------+
-   | This source file is subject to version 3.01 of the PHP license,      |
+   | This source file is subject to version 3.0 of the PHP license,       |
    | that is bundled with this package in the file LICENSE, and is        |
    | available through the world-wide-web at the following url:           |
-   | http://www.php.net/license/3_01.txt                                  |
+   | http://www.php.net/license/3_0.txt.                                  |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -17,9 +17,7 @@
    +----------------------------------------------------------------------+
  */
 
-
-/* $Id: php_main.h,v 1.23.2.2.2.1 2006/01/01 13:46:59 sniper Exp $ */
-
+/* $Id: php_main.h,v 1.31.2.1 2005/01/09 16:30:15 sniper Exp $ */
 
 #ifndef PHP_MAIN_H
 #define PHP_MAIN_H
@@ -28,6 +26,7 @@
 #include "php_globals.h"
 #include "SAPI.h"
 
+BEGIN_EXTERN_C()
 PHPAPI int php_request_startup(TSRMLS_D);
 PHPAPI void php_request_shutdown(void *dummy);
 PHPAPI void php_request_shutdown_for_exec(void *dummy);
@@ -35,10 +34,12 @@ PHPAPI int php_module_startup(sapi_module_struct *sf, zend_module_entry *additio
 PHPAPI void php_module_shutdown(TSRMLS_D);
 PHPAPI void php_module_shutdown_for_exec(void);
 PHPAPI int php_module_shutdown_wrapper(sapi_module_struct *sapi_globals);
+PHPAPI int php_request_startup_for_hook(TSRMLS_D);
 
 PHPAPI int php_startup_extensions(zend_module_entry **ptr, int count);
 
 PHPAPI int php_execute_script(zend_file_handle *primary_file TSRMLS_DC);
+PHPAPI int php_execute_simple_script(zend_file_handle *primary_file, zval **ret TSRMLS_DC);
 PHPAPI int php_handle_special_queries(TSRMLS_D);
 PHPAPI int php_lint_script(zend_file_handle *file TSRMLS_DC);
 
@@ -53,5 +54,6 @@ extern void php_free_shutdown_functions(TSRMLS_D);
 /* environment module */
 extern int php_init_environ(void);
 extern int php_shutdown_environ(void);
+END_EXTERN_C()
 
 #endif
