@@ -1,13 +1,13 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 4                                                        |
+   | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2006 The PHP Group                                |
+   | Copyright (c) 1997-2004 The PHP Group                                |
    +----------------------------------------------------------------------+
-   | This source file is subject to version 3.01 of the PHP license,      |
+   | This source file is subject to version 3.0 of the PHP license,       |
    | that is bundled with this package in the file LICENSE, and is        |
    | available through the world-wide-web at the following url:           |
-   | http://www.php.net/license/3_01.txt                                  |
+   | http://www.php.net/license/3_0.txt.                                  |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -16,11 +16,17 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: php_functions.c,v 1.31.2.2.8.1 2006/01/01 13:47:01 sniper Exp $ */
+/* $Id: php_functions.c,v 1.41 2004/01/08 08:18:04 andi Exp $ */
+
+
+#define ZEND_INCLUDE_FULL_WINDOWS_HEADERS
 
 #include "php.h"
+#include "ext/standard/php_smart_str.h"
+#include "ext/standard/info.h"
 #include "SAPI.h"
 
+#define CORE_PRIVATE
 #include "apr_strings.h"
 #include "apr_time.h"
 #include "ap_config.h"
@@ -153,9 +159,7 @@ PHP_FUNCTION(apache_request_headers)
 	const apr_array_header_t *arr;
 	char *key, *val;
 
-	if (array_init(return_value) == FAILURE) {
-		RETURN_FALSE;
-	}
+	array_init(return_value);
 	
 	ctx = SG(server_context);
 	arr = apr_table_elts(ctx->f->r->headers_in);
@@ -175,9 +179,7 @@ PHP_FUNCTION(apache_response_headers)
 	const apr_array_header_t *arr;
 	char *key, *val;
 
-	if (array_init(return_value) == FAILURE) {
-		RETURN_FALSE;
-	}
+	array_init(return_value);
 	
 	ctx = SG(server_context);
 	arr = apr_table_elts(ctx->f->r->headers_out);
@@ -286,8 +288,72 @@ PHP_FUNCTION(apache_getenv)
 }
 /* }}} */
 
+static char *php_apache_get_version()
+{
+	return (char *) ap_get_server_version();
+}
+
+/* {{{ proto string apache_get_version(void)
+   Fetch Apache version */
+PHP_FUNCTION(apache_get_version)
+{
+	char *apv = php_apache_get_version();
+
+	if (apv && *apv) {
+		RETURN_STRING(apv, 1);
+	} else {
+		RETURN_FALSE;
+	}
+}
+/* }}} */
+
+/* {{{ proto array apache_get_modules(void)
+   Get a list of loaded Apache modules */
+PHP_FUNCTION(apache_get_modules)
+{
+	int n;
+	char *p;
+	
+	array_init(return_value);
+	
+	for (n = 0; ap_loaded_modules[n]; ++n) {
+		char *s = (char *) ap_loaded_modules[n]->name;
+		if ((p = strchr(s, '.'))) {
+			add_next_index_stringl(return_value, s, (p - s), 1);
+		} else {
+			add_next_index_string(return_value, s, 1);
+		}
+	}
+}
+/* }}} */
+
 PHP_MINFO_FUNCTION(apache)
 {
+	char *apv = php_apache_get_version();
+	smart_str tmp1 = {0};
+	int n;
+	char *p;
+	
+	for (n = 0; ap_loaded_modules[n]; ++n) {
+		char *s = (char *) ap_loaded_modules[n]->name;
+		if ((p = strchr(s, '.'))) {
+			smart_str_appendl(&tmp1, s, (p - s));
+		} else {
+			smart_str_appends(&tmp1, s);
+		}
+		smart_str_appendc(&tmp1, ' ');
+	}
+	if ((tmp1.len - 1) >= 0) {
+		tmp1.c[tmp1.len - 1] = '\0';
+	}
+            
+	php_info_print_table_start();
+	if (apv && *apv) {
+		php_info_print_table_row(2, "Apache Version", apv);
+	}
+	php_info_print_table_row(2, "Loaded Modules", tmp1.c);
+	smart_str_free(&tmp1);
+	php_info_print_table_end();
 }
 
 static function_entry apache_functions[] = {
@@ -298,6 +364,8 @@ static function_entry apache_functions[] = {
 	PHP_FE(apache_setenv, NULL)
 	PHP_FE(apache_getenv, NULL)
 	PHP_FE(apache_note, NULL)
+	PHP_FE(apache_get_version, NULL)
+	PHP_FE(apache_get_modules, NULL)
 	PHP_FALIAS(getallheaders, apache_request_headers, NULL)
 	{NULL, NULL, NULL}
 };
