@@ -9,7 +9,7 @@
  * See the file "LICENSE.TERMS" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * $Id: fcgiapp.h,v 1.1.4.2 2003/09/21 22:08:17 sas Exp $
+ * $Id: fcgiapp.h,v 1.2 2002/12/01 21:37:14 shane Exp $
  */
 
 #ifndef _FCGIAPP_H
@@ -24,7 +24,11 @@
 
 #ifndef DLLAPI
 #ifdef _WIN32
+#if defined(_LIB) || defined(FCGI_STATIC)
+#define DLLAPI
+#else
 #define DLLAPI __declspec(dllimport)
+#endif
 #else
 #define DLLAPI
 #endif
@@ -75,6 +79,19 @@ typedef struct FCGX_Stream {
  */
 typedef char **FCGX_ParamArray;
 
+/*
+ * A vector of pointers representing the parameters received
+ * by a FastCGI application server, with the vector's length
+ * and last valid element so adding new parameters is efficient.
+ */
+
+typedef struct Params {
+    FCGX_ParamArray vec;    /* vector of strings */
+    int length;		    /* number of string vec can hold */
+    char **cur;		    /* current item in vec; *cur == NULL */
+} Params;
+typedef Params *ParamsPtr;
+
 /*
  * FCGX_Request Flags
  *
@@ -94,19 +111,18 @@ typedef struct FCGX_Request {
     FCGX_Stream *in;
     FCGX_Stream *out;
     FCGX_Stream *err;
-    char **envp;
+	FCGX_ParamArray envp;
 
-    /* Don't use anything below here */
+	/* Don't use anything below here */
 
-    struct Params *paramsPtr;
+    ParamsPtr paramsPtr;
     int ipcFd;               /* < 0 means no connection */
     int isBeginProcessed;     /* FCGI_BEGIN_REQUEST seen */
     int keepConnection;       /* don't close ipcFd at end of request */
     int appStatus;
     int nWriters;             /* number of open writers (0..2) */
-    int flags;
-    int listen_sock;
-    int detached;
+	int flags;
+	int listen_sock;
 } FCGX_Request;
 
 
@@ -617,15 +633,6 @@ DLLAPI void FCGX_FreeStream(FCGX_Stream **stream);
  */
 DLLAPI void FCGX_ShutdownPending(void);
 
-
-/*
- *  Attach/Detach an accepted request from its listen socket.
- *  XXX This is not fully implemented at this time (patch welcome).
- */
-DLLAPI int FCGX_Attach(FCGX_Request * r);
-DLLAPI int FCGX_Detach(FCGX_Request * r);
-
-
 #if defined (__cplusplus) || defined (c_plusplus)
 } /* terminate extern "C" { */
 #endif
