@@ -16,10 +16,18 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: php_getopt.h,v 1.5 2004/01/08 17:33:23 sniper Exp $ */
+/* $Id: php_getopt.h,v 1.5.2.1 2005/03/11 11:04:03 hyanantha Exp $ */
 
 #include "php.h"
 
+#ifdef NETWARE
+/*
+As NetWare LibC has optind and optarg macros defined in unistd.h our local variables were getting mistakenly preprocessed so undeffing optind and optarg
+*/
+#undef optarg
+#undef optind
+#endif
+
 /* Define structure for one recognized option (both single char and long name).
  * If short_open is '-' this is the last option.
  */
